// Bibliothek zur Ansteuerung der Servos
#include <Servo.h>

//      Funktion  Pin  Beschreibung
#define IR_SENSOR 2    // Der Infrarotsesor ist an Pin 2
#define DIR_A     12   // Richtung Motor A
#define DIR_B     13   // Richtung Motor B
#define PWM_A     3    // PWM Motor A
#define PWM_B     11   // PWM Motor B
#define BRAKE_A   9    // Bremse Motor A
#define BRAKE_B   8    // Bremse Motor B
#define SERVO     5    // Servo wird an Pin 5 angesteuert


#define FULL_SPEED        125     // 255 ist Vollgas. Langsamer fahren ist bei diesem Programm schlauer.
#define LEFT              LOW     // Links drehen bei LOW
#define RIGHT             HIGH    // ... und rechts bei HIGH
#define HAND_DISTANCE     350     // Ab diesem Wert am IR-Sensor soll der Roboter die Hand erkennen -> Ist der Wert KLEINER, ist das Hand weiter weg.
                                  // Es funktionieren Sensorwerte ungefähr zwischen 50 und 600.
#define TOO_CLOSE         500     // Ab diesem Wert am IR-Sensor soll der Roboter stoppen, damit er nicht an eine Wand fährt.
#define SERVO_LEFT         45     // Bei welchem Winkel stößt der Sensor auf dem Servo links an?   < <  Ggf. anpassen!
#define SERVO_MIDDLE       95     // Bei diesem Wert schaut der Roboter genau geradeaus.           < <  Ggf. anpassen!
#define SERVO_RIGHT       180     // Bei welchem Winkel stößt der Sensor auf dem Servo rechts an?  < <  Ggf. anpassen!

// Werte, wenn am Netzteil angeschlossen!!
//#define TURN_SPEED         50     // Sehr langsame Geschwindigkeit während des Drehens!
//#define SERVO_DELAY        10     // Bestimmt die Verzögerung beim Drehen des Roboters in der Funktion drehen().

// Werte bei Akku-Stromversorgung!!
#define TURN_SPEED         90     // Sehr langsame Geschwindigkeit während des Drehens!
#define SERVO_DELAY        15     // Bestimmt die Verzögerung beim Drehen des Roboters in der Funktion drehen().


Servo SensorServo;                    // Mit diesem Objekt wird der Servo gesteuert
byte servoPosition = SERVO_MIDDLE;    // Mittelposition des Servos (geradeaus)
boolean servoRichtung = RIGHT;        // Drehrichtung des Servos
int sensorValue = 0;                  // Variable für den IR-Sensor


// Sucht nach einer Hand vor dem IR-Sensor. Dreht den Kopf (Sensor) dabei hin und her)
int handSuchen()
{
  int handPosition = 0;


  // Erst mal Roboter anhalten
  digitalWrite(BRAKE_A, HIGH);
  digitalWrite(BRAKE_B, HIGH);

  // Servo zu Beginn ganz nach RECHTS drehen
  SensorServo.write(SERVO_RIGHT);

  // Solange Distanz zum nächsten Objekt zu groß ist, schwenke Servo hin und her.
  do
  {
    //------------------------------------ 
    // Servo von rechts nach links drehen
    //------------------------------------ 
    for (int servoPosition = SERVO_RIGHT; servoPosition >= SERVO_LEFT; servoPosition--)
    {
      // Servo um einen Schritt drehen
      SensorServo.write(servoPosition);
  
      // Warten bis Servoposition erreicht
      delay(10);
      
      // Sensor auslesen um Hand zu finden
      // Kleiner Wert bedeutet große Distanz. Großer Wert = geringe Distanz!
      sensorValue = analogRead(IR_SENSOR);
  
      // Ist der gemessene Wert kleiner, als der minimale Wert?
      // Sprich: Ist Hand nah genug dran?
      if (sensorValue >= HAND_DISTANCE)
      {
        // Ja: An dieser Position ist ein Objekt (Hand) am nächsten.
        // Merken wir uns die Servo- bzw. Hand-Position.
        handPosition = servoPosition;
        
        // Hand entdeckt. Position zurückgeben
        return handPosition;
      }
    }
  
    //------------------------------------ 
    // Servo von links nach rechts drehen
    //------------------------------------ 
    for (int servoPosition = SERVO_LEFT; servoPosition <= SERVO_RIGHT; servoPosition++)
    {
      // Servo um einen Schritt drehen
      SensorServo.write(servoPosition);
  
      // Warten bis Servoposition erreicht
      delay(10);
      
      // Sensor auslesen um Hand zu finden
      // Kleiner Wert bedeutet große Distanz. Großer Wert = geringe Distanz!
      sensorValue = analogRead(IR_SENSOR);
  
      // Ist der gemessene Wert kleiner, als der minimale Wert?
      // Sprich: Ist Hand nah genug dran?
      if (sensorValue >= HAND_DISTANCE)
      {
        // Ja: gemessener Wert ist der, wo ein Objekt (Hand) am nächsten ist.
        // Merken wir uns die Servo- bzw. Hand-Position.
        handPosition = servoPosition;
        
        // Hand entdeckt. Position zurückgeben
        return handPosition;
      }
    }
  
  } while (sensorValue < HAND_DISTANCE);
  // solange Hand zu weit weg vom Sensor

  // Diese Zeile wird eigentlich nie erreicht...
  // Korrekterweise müsste hier ein "return" stehen. Arduino akzeptiert das aber.
}


// Vorwärts fahren und die Distanz zur Hand prüfen
void fahreGeradeaus()
{
  // Zur Sicherheit noch mal: Servo geradeaus
  SensorServo.write(SERVO_MIDDLE);

  // Beide Motoren auf geradeaus stellen
  digitalWrite(DIR_A, HIGH);
  digitalWrite(DIR_B, HIGH);
  
  // Geschwindigkeit setzen
  analogWrite(PWM_A, FULL_SPEED);
  analogWrite(PWM_B, FULL_SPEED);
  
  // Losfahren!
  digitalWrite(BRAKE_A, LOW);
  digitalWrite(BRAKE_B, LOW);

  //----------------------------------------------------
  // Fahre die ganze Zeit weiter, solange Hand in Sicht
  //----------------------------------------------------
  do
  {
    // Sensor auslesen um Hand zu finden
    // Kleiner Wert bedeutet große Distanz. Großer Wert = geringe Distanz!
    sensorValue = analogRead(IR_SENSOR);

    // kleine Pause
    delay(10);

  } while ((sensorValue >= HAND_DISTANCE - 30) && (sensorValue <= TOO_CLOSE));
  // solange Hand nah genug dran am Sensor (-30 um ganz sicher zu gehen)
  // und (&&) die Hand nicht zu dicht dran ist!

  //--------------------------------------
  // Hand ist weg oder zu nah dran!
  //--------------------------------------
  // Anhalten!
  digitalWrite(BRAKE_A, HIGH);
  digitalWrite(BRAKE_B, HIGH);
  
  // Kurz innehalten
  delay(1000);
}


// Lässt den Roboter nach links (LEFT) oder rechts (RIGHT) drehen
// Zusätzlich wird die Position der Hand (des Servos) übergeben.
void drehen(boolean Direction, int servoPos)
{
  // Bremsen
  digitalWrite(BRAKE_A, HIGH); 
  digitalWrite(BRAKE_B, HIGH);

  // Motor A in entgegen der "RICHTUNG" drehen
  digitalWrite(DIR_A, !Direction);
  // Motor B in die "RICHTUNG" drehen
  digitalWrite(DIR_B, Direction);
   
  // Geschwindigkeit für das Drehen einstellen
  analogWrite(PWM_A, TURN_SPEED);
  analogWrite(PWM_B, TURN_SPEED);
  
  do
  {
    // Bremsen lösen
    digitalWrite(BRAKE_A, LOW);
    digitalWrite(BRAKE_B, LOW);
    
    // Servo um eine Position drehen
    SensorServo.write(servoPos);

    // Roboter links rum drehen
    if (Direction == LEFT)
    {
      // Servo nach rechts drehen      
      servoPos++;
    }
    else
    {
      // Roboter rechts rum drehen
      // Servo nach links drehen      
      servoPos--;
    }

    // Wichtige Verzögerung, welche die Drehgeschwindigkeit bestimmt
    // und den Servo an seine Position fahren lässt!!
    delay(SERVO_DELAY);

    // Anhalten
    digitalWrite(BRAKE_A, HIGH);
    digitalWrite(BRAKE_B, HIGH);

  } while (servoPos != SERVO_MIDDLE);
  // Bis Servo Mittelposition erreicht hat (und Roboter sich dabei mitgedreht hat)
  
  // Halt!
  digitalWrite(BRAKE_A, HIGH); 
  digitalWrite(BRAKE_B, HIGH);
}


void setup( )
{
  // Pins für Motorrichtungen als Ausgang definieren
  pinMode(DIR_A, OUTPUT);
  pinMode(DIR_B, OUTPUT);

  // Pins für Bremsen als Ausgang definieren
  pinMode(BRAKE_A, OUTPUT);
  pinMode(BRAKE_B, OUTPUT);
  
  // Beide Bremsen anziehen, HIGH = Bremsen!
  digitalWrite(BRAKE_A, HIGH);
  digitalWrite(BRAKE_B, HIGH);

  // Servo initialiseren
  SensorServo.attach(SERVO);

  // Servo zu Beginn in die Mitte drehen
  SensorServo.write(SERVO_MIDDLE);

  // Zur Sicherheit eine Sekunde warten
  delay(1000);
}


void loop()
{
  int handRichtung = 0; // Position der entdeckten Hand


  // Zur Sicherheit noch mal: Servo geradeaus
  SensorServo.write(SERVO_MIDDLE);

   // Hand vor Sensor suchen und gefundene Servoposition speichern
  handRichtung = handSuchen();

  // Roboter Richtung Hand drehen
  if (handRichtung <= SERVO_MIDDLE)
  {
    // Nach links drehen
    drehen(LEFT, handRichtung);
  }
  else
  {
    // Nach rechts drehen
    drehen(RIGHT, handRichtung);
  }
 
  // Kurz innehalten
  delay(1000);

  // Fahre geradeaus (Richtung zuvor erkannter Hand)
  fahreGeradeaus();
}

